
object TreeTest {

  def testrange(t: BinarySearchTree, low: Int, hi: Int) {
    printf("Range [%d,%d] = ", low, hi)
    println(t.range(low, hi))
  }

  def testupper(t: BinarySearchTree, k: Int) {
    printf("Upper(%d) = ", k)
    try {
      println(t.upperNeighbor(k))
    } catch {
      case e: NoSuchElementException => 
	println("No such neighbor")
    }
  }

  def testsplit(t: BinarySearchTree, x: Int) {
    printf("Splitting at %d:\n", x)
    val (t1, t2) = t.split(x)
    printf("Left size: %d, height %d, max key: %s\n",
	   t1.size, t1.height, 
	   if (t1.isEmpty) "?" else t1.lastKey)
    printf("Right size: %d, height %d, min key: %s\n",
	   t2.size, t2.height, 
	   if (t2.isEmpty) "?" else t2.firstKey)
  }

  def test(nums: Int, gap: Int) {

    var t = new BinarySearchTree

    println("Inserting numbers into tree.")
    var i = gap % nums
    while (i != 0) {
      t = t + (i -> i)
      i = (i + gap) % nums
    }

    println("Height of tree t is " + t.height)
    println("Size of tree t is " + t.size)

    print("\nRemoving odd numbers from tree:")
    for (i <- 1 until nums by 2)
      t = t - i
    
    println("\nHeight of tree is " + t.height)
    println("Size of tree t is " + t.size)

    printf("First key: %d, last key: %d\n", t.firstKey, t.lastKey)
    
    if (t.firstKey != 2 || t.lastKey != nums - 2)
      println("firstKey/lastKey error!")
    
    println("Testing that all numbers are in the map:")
    for (i <- 2 until nums by 2) {
      if (!(t contains i) || t(i) != i)
	printf("Error: %d not in map or not mapping to itself\n", i)
    }
    
    println("Testing that numbers were deleted:")
    for (i <- 1 until nums by 2) {
      if (t contains i)
	printf("Error: %d still in the map\n", i)
    }

    println("Computing ranges:")
    testrange(t, -7, 19)
    testrange(t, 16, 23)
    testrange(t, 345, 123)
    testrange(t, nums - 7, nums + 5)

    println("Upper neighbors:")
    testupper(t, nums - 1)
    testupper(t, nums - 2)
    testupper(t, -7)
    testupper(t, 15)
    testupper(t, 64)

    println("Splitting:")
    testsplit(t, 13)
    testsplit(t, -5)
    testsplit(t, nums / 2)
    testsplit(t, nums - 2)
    testsplit(t, nums - 3)
  }

  def main(args: Array[String]) {
    if (args.length != 2) {
      println("Usage: scala TreeTest <num> <gap>")
      sys.exit(1)
    }

    val n = args(0).toInt
    val gap = args(1).toInt
    test(n, gap)
  }
}
